/**********************************************************************************************

Author : Mauro Laurenti
Version : 1.0
Date : 26/05/2012
 
CopyRight 2006-2012 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) 
is intended for use solely and exclusively on Microchip PIC Microcontroller (registered mark).  
The software is owned by the Author, and is protected under applicable copyright laws. 
All rights are reserved. 
Any use in violation of the foregoing restrictions may subject the 
user to criminal sanctions under applicable laws (Italian or International ones), as well as to 
civil liability for the breach of the terms and conditions of this license. 
Commercial use is forbidden without a written acknowledgment with the Author.
Personal or educational use is allowed if the application containing the following 
software doesn't aim to commercial use or monetary earning of any kind.    

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT, 
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

**********************************************************************************************/
									

#include <GLCD_KS0108B.h>
#include <delays.h>


unsigned char font_width = 0;
unsigned char font_height = 0;


//*********************************************
//             get_busy_status_GLCD
//*********************************************
void wait_busy_status_GLCD (void){

	unsigned char busy_flag = 1;

	while (busy_flag) {

		if (get_controller_status_GLCD () & GLCD_BUSY_FLAG) 
			busy_flag = 1;
		else
			busy_flag = 0;
	}
}


//*********************************************
//             get_controller_status_GLCD
//*********************************************
unsigned char get_controller_status_GLCD (void) {

	unsigned char controller_status; 

	GLCD_DATA_DIRECTION = INPUT;

	GLCD_E = LOW;
	Delay1TCY () ;
	GLCD_R_W = HIGH;
	GLCD_D_I = LOW;
	Delay1TCY ();
	GLCD_E = HIGH;
	Delay1TCY ();
	controller_status = GLCD_DATA_READ;
	
	GLCD_DATA_DIRECTION = OUTPUT;

	return (controller_status);
}


//*********************************************
//             write_command_GLCD
//*********************************************
void write_command_GLCD (unsigned char command_to_send) {

	GLCD_E = LOW;
	Delay1TCY ();
	Delay1TCY ();
	GLCD_R_W = LOW;
	GLCD_D_I = LOW;
	Delay1TCY ();

	GLCD_E = HIGH;
	GLCD_DATA_WRITE = command_to_send;
	Delay1TCY ();
	GLCD_E = LOW;

	Delay1TCY ();
	GLCD_R_W = HIGH;
	GLCD_D_I = LOW;

	Delay1TCY ();

	GLCD_E = HIGH;
}


//*********************************************
//             set_display_GLCD
//*********************************************
void set_display_GLCD (unsigned char display_status){
	
	GLCD_CS1 = HIGH;
	GLCD_CS2 = LOW;

	wait_busy_status_GLCD ();
	if (display_status)
		write_command_GLCD (COMM_DISPLAY_ON);
	else 
		write_command_GLCD (COMM_DISPLAY_OFF);

	wait_busy_status_GLCD ();

	GLCD_CS1 = LOW;
	GLCD_CS2 = HIGH;

	wait_busy_status_GLCD ();
	if (display_status)
		write_command_GLCD (COMM_DISPLAY_ON);
	else 
		write_command_GLCD (COMM_DISPLAY_OFF);

	wait_busy_status_GLCD ();

	GLCD_CS1 = LOW;
	GLCD_CS2 = LOW;

}

//*********************************************
//             set_page_GLCD
//*********************************************
void set_page_GLCD (unsigned char controller_page) {

	wait_busy_status_GLCD ();
	write_command_GLCD (COMM_PAGE_SELECT | controller_page);
	wait_busy_status_GLCD ();
}


//*********************************************
//             set_address_GLCD
//*********************************************
void set_address_GLCD (unsigned char controller_address){

	wait_busy_status_GLCD ();
	write_command_GLCD (COMM_ADDRESS_SELECT | controller_address);
	wait_busy_status_GLCD ();
}


//*********************************************
//             set_vertical_offset_GLCD
//*********************************************
void set_vertical_offset_GLCD (unsigned char vertical_offset){

	GLCD_CS1 = HIGH;
	GLCD_CS2 = LOW;

//	wait_busy_status_GLCD ();
	write_command_GLCD (COMM_START_LINE | vertical_offset);
//	wait_busy_status_GLCD ();

	GLCD_CS1 = LOW;
	GLCD_CS2 = HIGH;

//	wait_busy_status_GLCD ();
	write_command_GLCD (COMM_START_LINE | vertical_offset);
//	wait_busy_status_GLCD ();

	GLCD_CS1 = LOW;
	GLCD_CS2 = LOW;

}


//*********************************************
//             write_data_GLCD
//*********************************************
void write_data_GLCD (unsigned char data_to_send) {

	GLCD_E = LOW;
	Delay1TCY ();
	Delay1TCY ();
	GLCD_R_W = LOW;
	GLCD_D_I = HIGH;
	Delay1TCY ();
	Delay1TCY ();
	GLCD_E = HIGH;
	GLCD_DATA_WRITE = data_to_send;
	Delay1TCY ();
	Delay1TCY ();
	Delay1TCY ();
	GLCD_E = LOW;

	Delay1TCY ();
	GLCD_R_W = HIGH;
	GLCD_D_I = LOW;

	Delay1TCY ();

	GLCD_E = HIGH;

}


//*********************************************
//             clear_GLCD
//*********************************************
void clear_GLCD (unsigned char filling_color) {
	
	unsigned char page = 0;
	unsigned char address = 0;

	GLCD_CS1 = HIGH;
	GLCD_CS2 = LOW;
	
	set_address_GLCD (address);

	for (page = 0; page < NUMBER_OF_PAGES; page++) {
		set_page_GLCD (page);
		for (address = 0; address < Y_RESOLUTION; address++) {

		//	wait_busy_status_GLCD ();
			write_data_GLCD (filling_color);
		//	wait_busy_status_GLCD ();
		}
	}

	GLCD_CS1 = LOW;
	GLCD_CS2 = HIGH;


	address = 0;
	set_address_GLCD (address);

	for (page = 0; page < NUMBER_OF_PAGES; page++) {
		set_page_GLCD (page);
		
		for (address = 0; address < Y_RESOLUTION; address++) {
		//	wait_busy_status_GLCD ();
			write_data_GLCD (filling_color);
		//	wait_busy_status_GLCD ();
		}
	}

	GLCD_CS1 = LOW;
	GLCD_CS2 = LOW;

}

//*********************************************
//             read_data_GLCD
//*********************************************
unsigned char read_data_GLCD (void) {

	unsigned char data_out; 

	GLCD_DATA_DIRECTION = INPUT;

	GLCD_E = LOW;
	Delay1TCY ();
	Delay1TCY ();
	GLCD_R_W = HIGH;
	GLCD_D_I = HIGH;
	Delay1TCY ();
	GLCD_E = HIGH;
	Delay1TCY ();
	data_out = GLCD_DATA_READ;
	Delay1TCY ();
	Delay1TCY ();
	GLCD_E = LOW;
	Delay1TCY ();
	GLCD_R_W = LOW;
	GLCD_D_I = LOW;	
	
	GLCD_DATA_DIRECTION = OUTPUT;

	return (data_out);
}

//*********************************************
//             plot_xy_GLCD
//*********************************************
void plot_xy_GLCD (unsigned char x_pos,  unsigned char y_pos, unsigned char filling_color){

	unsigned char data_byte;
	unsigned char data_mask;

	y_pos = Y_RESOLUTION-y_pos -1;

	if (x_pos < CONTROLLER_MAX_X) {

		GLCD_CS1 = HIGH;
		GLCD_CS2 = LOW;
		set_address_GLCD (x_pos);
		set_page_GLCD (y_pos >> 3);	
		//dummy read
		data_byte = read_data_GLCD ();
		data_byte = read_data_GLCD ();

		set_address_GLCD (x_pos);

	} else {
		
		GLCD_CS1 = LOW;
		GLCD_CS2 = HIGH;
		set_address_GLCD (x_pos-CONTROLLER_MAX_X);
		set_page_GLCD (y_pos >> 3);	
		//dummy read
		data_byte = read_data_GLCD ();
		data_byte = read_data_GLCD ();

		set_address_GLCD (x_pos-CONTROLLER_MAX_X);
	}

	data_mask = 0x01 << (y_pos & 0x07);	

	if (filling_color == FILLING_BLACK)
		data_byte = data_byte | data_mask;
	else
		data_byte = data_byte & ~data_mask;
	
//	wait_busy_status_GLCD ();
	write_data_GLCD (data_byte);
//	wait_busy_status_GLCD ();
	

	GLCD_CS1 = LOW;
	GLCD_CS2 = LOW;
}


//*********************************************
//           draw_vertical_line_GLCD
//*********************************************
void draw_vertical_line_GLCD (unsigned char x_origin, unsigned char y_origin, unsigned char length, unsigned char filling_color){

	unsigned char i;

	for (i = 0 ; i<length; i++) {
		plot_xy_GLCD (x_origin, y_origin + i, filling_color);
	}
}


//*********************************************
//          draw_horizontal_line_GLCD
//*********************************************
void draw_horizontal_line_GLCD (unsigned char x_origin, unsigned char y_origin, unsigned char length, unsigned char filling_color){

	unsigned char i;

	for (i = 0 ; i<length; i++) {
		plot_xy_GLCD (x_origin + i, y_origin, filling_color);
	}
}


//*********************************************
//             draw_window_GLCD
//*********************************************

void draw_window_GLCD (unsigned char x_origin, unsigned char y_origin, unsigned char width, unsigned char height, unsigned char filling_color){

	unsigned char i;

	draw_horizontal_line_GLCD (x_origin, y_origin, width, filling_color);
	// + 1 is required to close the window
	draw_horizontal_line_GLCD (x_origin, y_origin + height, width+1, filling_color);

	draw_vertical_line_GLCD (x_origin, y_origin, height, filling_color);
	draw_vertical_line_GLCD (x_origin + width, y_origin, height, filling_color);

}


//*********************************************
//             draw_box_GLCD
//*********************************************

void draw_box_GLCD (unsigned char x_origin, unsigned char y_origin, unsigned char width, unsigned char height, unsigned char filling_color){

	unsigned char i;

	for (i=0; i < width; i++)
		draw_vertical_line_GLCD (x_origin +i, y_origin, height, filling_color);

}

//*********************************************
//             write_char_GLCD
//*********************************************
void write_char_GLCD (unsigned char x_pos, unsigned char y_pos, unsigned char character){

	unsigned char i;
	unsigned char data;
	unsigned char x;
	unsigned char bit_value;
	int data_index; 

	for (i = 0 ; i<font_width; i++){

		data = font_5x7[character - 32][i];

		for (x = 0 ; x<= font_height; x++) {
			
			if (data & (0x80 >> x))
				bit_value = FILLING_BLACK;
			else 
				bit_value = FILLING_WHITE;

			plot_xy_GLCD (x_pos + i, y_pos + x, bit_value);
			
		}			
	}	
}

//*********************************************
//             write_string_GLCD
//*********************************************

void write_string_GLCD (unsigned char x_pos, unsigned char y_pos, unsigned char * character_array){

unsigned char i = 0;

	while (character_array[i]) {
		
		write_char_GLCD (x_pos + (i* (font_width+ GLCD_FONT_SPACE)) , y_pos,character_array[i]);
	
		i++;
	}
}


//*********************************************
//             write_message_GLCD
//*********************************************
void write_message_GLCD (unsigned char x_pos, unsigned char y_pos, const rom unsigned char * character) {

unsigned char i = 0;

    while (character[i]) {

        write_char_GLCD (x_pos + (i*(font_width + GLCD_FONT_SPACE)), y_pos,character[i]);

        i++;
    }

}


//*********************************************
//             draw_picture_GLCD
//*********************************************
void draw_picture_GLCD (rom unsigned char  * picture_table){

	unsigned char page = 0;
	unsigned char address = 0;
	

	for (page = 0; page < NUMBER_OF_PAGES; page++) {
	GLCD_CS1 = HIGH;
	GLCD_CS2 = LOW;
	
	set_address_GLCD (address);
	set_page_GLCD (page);
	
	for (address = 0; address < Y_RESOLUTION; address++) {

		write_data_GLCD (* picture_table);
		picture_table++;
	}

	GLCD_CS1 = LOW;
	GLCD_CS2 = HIGH;

	address = 0;
	set_address_GLCD (address);
	set_page_GLCD (page);
		
	for (address = 0; address < Y_RESOLUTION; address++) {

		write_data_GLCD (* picture_table);
		picture_table++;
	}

	}

	GLCD_CS1 = LOW;
	GLCD_CS2 = LOW;
}


//*********************************************
//             write_integer_GLCD
//*********************************************
void write_integer_GLCD (unsigned char x_pos, unsigned char y_pos, int value, char number_of_digits){

	// The array size is 5 plus end of string \0
	unsigned char convertedInt [6];
	
	// Index used to shift to the right the digit
	char index; 
	
	// Integer is converted to string
	itoa (value, (char*) convertedInt);	
	
	if (number_of_digits >0 ) {
		
		convertedInt[number_of_digits] = '\0';
		
		// Shift the digit to the right removing the empty one
		while (!isdigit(convertedInt[number_of_digits-1])) {
			
			for (index = number_of_digits-1; index > 0; index--){
				convertedInt[index] = convertedInt[index-1];
				convertedInt[index-1] = ' ';
			}							
		}	
	}

	write_string_GLCD (x_pos, y_pos, convertedInt);
}


//*********************************************
//             set_font_GLCD
//*********************************************
void set_font_GLCD (unsigned char width, unsigned char height){

	font_width = width;
	font_height = height;
}


//*********************************************
//             backlight_GLCD
//*********************************************
void backlight_GLCD(unsigned char backlight) {

	GLCD_LED = backlight;
}


//*********************************************
//             initialize_GLCD
//*********************************************
void  initialize_GLCD (void) {

	GLCD_DATA_DIRECTION = OUTPUT;

	GLCD_RST = LOW;
	GLCD_LED = LOW;
	GLCD_E = HIGH;
	GLCD_R_W = HIGH;
	GLCD_D_I = LOW;
	GLCD_CS1 = LOW;
	GLCD_CS2 = LOW;
	
	delay_ms (100);

	GLCD_RST = HIGH;
	
	backlight_GLCD (TURN_OFF_LED);
    set_vertical_offset_GLCD (0);
	clear_GLCD (FILLING_WHITE);
	set_display_GLCD (TURN_ON_DISPLAY);
	set_font_GLCD (GLCD_FONT_WIDTH_5, GLCD_FONT_HEIGTH_7);

}



